@@ -7,8 +7,10 @@
 open Support.Common
 open Gtk_common
 open Zeroinstall.General
+open Zeroinstall
 
 module FeedAttr = Zeroinstall.Constants.FeedAttr
+module Impl = Zeroinstall.Impl
 module F = Zeroinstall.Feed
 module FC = Zeroinstall.Feed_cache
 module U = Support.Utils
@@ -41,7 +43,7 @@ let add_description_text config ~trust_db ~heading_style ~link_style (buffer:GTe
   let iter = buffer#start_iter in
   buffer#insert ~iter ~tags:[heading_style] feed.F.name;
   buffer#insert ~iter (spf " (%s)" (default "-" @@ F.get_summary config.langs feed));
-  buffer#insert ~iter (spf "\n%s\n" (Zeroinstall.Feed_url.format_url feed.F.url));
+  buffer#insert ~iter (spf "\n%s\n" (Feed_url.format_url feed.F.url));
 
   if description.times <> [] then (
     buffer#insert ~iter "\n";
@@ -77,7 +79,7 @@ let add_description_text config ~trust_db ~heading_style ~link_style (buffer:GTe
   | `remote_feed _ as feed_url, sigs ->
       let module G = Support.Gpg in
       buffer#insert ~iter ~tags:[heading_style] "\nSignatures\n";
-      let domain = Zeroinstall.Trust.domain_from_url feed_url in
+      let domain = Trust.domain_from_url feed_url in
       sigs |> Lwt_list.iter_s (function
         | G.ValidSig {G.fingerprint; G.timestamp} ->
             lwt name = G.get_key_name config.system fingerprint in
@@ -100,7 +102,7 @@ let generate_feed_description config trust_db feed overrides =
     | `remote_feed _ as feed_url ->
         lwt sigs = get_sigs config feed_url in
         if sigs <> [] then (
-          let domain = Zeroinstall.Trust.domain_from_url feed_url in
+          let domain = Trust.domain_from_url feed_url in
           match trust_db#oldest_trusted_sig domain sigs with
           | Some last_modified -> times := ("Last upstream change", last_modified) :: !times
           | None -> ()
@@ -125,8 +127,9 @@ let generate_feed_description config trust_db feed overrides =
     | Some description -> Str.split (Str.regexp_string "\n\n") description |> List.map format_para
     | None -> ["-"] in
 
-  let homepages = feed.F.root |> ZI.map ~name:"homepage" (fun homepage ->
-    homepage.Q.last_text_inside;
+  let homepages = Element.feed_metadata feed.F.root |> U.filter_map (function
+    | `homepage homepage -> Some (Element.simple_content homepage)
+    | _ -> None
   ) in
 
   Lwt.return {
@@ -239,12 +242,12 @@ let add_remote_feed ~parent ~watcher ~recalculate tools iface () =
           try_lwt
             let url = entry#text in
             if url = "" then raise_safe "Enter a URL";
-            begin match Zeroinstall.Feed_url.parse_non_distro url with
+            begin match Feed_url.parse_non_distro url with
             | `local_feed _ -> raise_safe "Not a remote feed!"
             | `remote_feed _ as feed_url ->
                 let config = tools#config in
-                let fetcher = tools#make_fetcher (watcher :> Zeroinstall.Progress.watcher) in
-                lwt () = Zeroinstall.Gui.add_remote_feed config fetcher iface feed_url in
+                let fetcher = tools#make_fetcher (watcher :> Progress.watcher) in
+                lwt () = Gui.add_remote_feed config fetcher iface feed_url in
                 box#destroy ();
                 recalculate ~force:false;
                 Lwt.return () end
@@ -276,10 +279,10 @@ let add_local_feed ~parent ~recalculate config iface () =
     | `OK ->
         try
           let path = box#filename |? lazy (raise_safe "No filename!") in
-          match Zeroinstall.Feed_url.parse_non_distro path with
+          match Feed_url.parse_non_distro path with
           | `remote_feed _ -> raise_safe "Not a local feed!"
           | `local_feed _ as feed_url ->
-              Zeroinstall.Gui.add_feed config iface feed_url;
+              Gui.add_feed config iface feed_url;
               box#destroy ();
               recalculate ~force:false
         with Safe_exception _ as ex -> Alert_box.report_error ~parent:box ex
@@ -338,7 +341,7 @@ let make_feeds_tab tools ~trust_db ~recalculate ~watcher window iface =
     | [path] ->
         let iter = feeds_model#get_iter path in
         let feed_import = feeds_model#get ~row:iter ~column:feed_obj in
-        Zeroinstall.Gui.remove_feed config iface feed_import.F.feed_src;
+        Gui.remove_feed config iface feed_import.F.feed_src;
         remove_feed#misc#set_sensitive false;
         recalculate ~force:false;
     | _ -> log_warning "Impossible selection!"
@@ -412,7 +415,7 @@ let make_feeds_tab tools ~trust_db ~recalculate ~watcher window iface =
       let iface_config = watcher#feed_provider#get_iface_config iface in
       let extra_feeds = iface_config.FC.extra_feeds in
 
-      let master_feed = Zeroinstall.Feed_url.master_feed_of_iface iface in
+      let master_feed = Feed_url.master_feed_of_iface iface in
       let imported_feeds =
         match watcher#feed_provider#get_feed master_feed with
         | None -> []
@@ -432,8 +435,8 @@ let make_feeds_tab tools ~trust_db ~recalculate ~watcher window iface =
         let arch_value =
           match feed.F.feed_os, feed.F.feed_machine with
           | None, None -> ""
-          | os, machine -> Zeroinstall.Arch.format_arch os machine in
-        feeds_model#set ~row ~column:url (Zeroinstall.Feed_url.format_url feed.F.feed_src);
+          | os, machine -> Arch.format_arch os machine in
+        feeds_model#set ~row ~column:url (Feed_url.format_url feed.F.feed_src);
         feeds_model#set ~row ~column:arch arch_value;
         feeds_model#set ~row ~column:used (watcher#feed_provider#was_used feed.F.feed_src);
         feeds_model#set ~row ~column:feed_obj feed;
@@ -453,7 +456,7 @@ let build_stability_menu set_stability =
   GMenu.separator_item ~packing:menu#add () |> ignore_widget;
 
   [Preferred; Packaged; Stable; Testing; Developer; Buggy; Insecure] |> List.iter (fun stability ->
-    let label = F.format_stability stability |> String.capitalize in
+    let label = Impl.format_stability stability |> String.capitalize in
     let item = GMenu.menu_item ~packing:menu#add ~label () in
     item#connect#activate ==> (fun () -> set_stability (Some stability))
   );
@@ -498,8 +501,11 @@ let show_explanation_box ~parent iface version reason =
     box#show ()
   )
 
-let make_versions_tab config reqs ~recalculate ~watcher window iface =
+let re_lt = Str.regexp_string "<"
+
+let make_versions_tab config reqs ~recalculate ~watcher window role =
   let vbox = GPack.vbox () in
+  let iface = role.Solver.iface in
 
   (* Stability policy *)
   let table = GPack.table
@@ -521,7 +527,7 @@ let make_versions_tab config reqs ~recalculate ~watcher window iface =
     ~value:iface_config.FC.stability_policy
     ~to_string:(function
       | None -> "Use default setting"
-      | Some level -> F.format_stability level |> String.capitalize
+      | Some level -> Impl.format_stability level |> String.capitalize
     )
     ~callback:set_stability_policy
     ~tooltip:"Implementations at this stability level or higher will be used in preference to others. \
@@ -586,7 +592,7 @@ let make_versions_tab config reqs ~recalculate ~watcher window iface =
             let menu = GMenu.menu () in
             let stability_menu = GMenu.menu_item ~packing:menu#add ~label:"Rating" () in
             let submenu = build_stability_menu (fun stability ->
-              Zeroinstall.Gui.set_impl_stability config (F.get_id impl) stability;
+              Gui.set_impl_stability config (Impl.get_id impl) stability;
               recalculate ~force:false
             ) in
             stability_menu#set_submenu submenu;
@@ -596,16 +602,17 @@ let make_versions_tab config reqs ~recalculate ~watcher window iface =
               item#connect#activate ==> (fun () ->
                 U.xdg_open_dir config.system path
               ) in
-            begin match impl.F.impl_type with
+            begin match impl.Impl.impl_type with
             | `local_impl path -> add_open_item path
             | `cache_impl info ->
-                let path = Zeroinstall.Stores.lookup_maybe config.system info.F.digests config.stores in
+                let path = Stores.lookup_maybe config.system info.Impl.digests config.stores in
                 path |> if_some add_open_item
             | `package_impl _ -> () end;
 
             let explain = GMenu.menu_item ~packing:menu#add ~label:"Explain this decision" () in
             explain#connect#activate ==> (fun () ->
-              let reason = Zeroinstall.Diagnostics.justify_decision config watcher#feed_provider reqs iface (F.get_id impl) in
+              let {Solver.iface; source; _} = role in
+              let reason = Justify.justify_decision config watcher#feed_provider reqs iface ~source (Impl.get_id impl) in
               show_explanation_box ~parent:window iface version_str reason
             );
             menu#popup ~button:(B.button bev) ~time:(B.time bev);
@@ -618,60 +625,56 @@ let make_versions_tab config reqs ~recalculate ~watcher window iface =
     method update =
       model#clear ();
       let (_ready, result) = watcher#results in
-      match Zeroinstall.Gui.list_impls result iface with
-      | None -> view#misc#set_sensitive false
-      | Some (selected, impls) ->
-          view#misc#set_sensitive true;
+      let (selected, impls) = Gui.list_impls result role in
+      let get_overrides =
+        let cache = ref StringMap.empty in
+        fun feed ->
+          match !cache |> StringMap.find feed with
+          | Some result -> result
+          | None ->
+              let result = F.load_feed_overrides config (Feed_url.parse feed) in
+              cache := !cache |> StringMap.add feed result;
+              result in
 
-          let get_overrides =
-            let cache = ref StringMap.empty in
-            fun feed ->
-              match !cache |> StringMap.find feed with
-              | Some result -> result
-              | None ->
-                  let result = F.load_feed_overrides config (Zeroinstall.Feed_url.parse feed) in
-                  cache := !cache |> StringMap.add feed result;
-                  result in
+      impls |> List.iter (fun (impl, problem) ->
+        let from_feed = Impl.get_attr_ex FeedAttr.from_feed impl in
+        let impl_id = Impl.get_attr_ex FeedAttr.id impl in
+        let overrides = get_overrides from_feed in
+        let stability_value =
+          match StringMap.find impl_id overrides.F.user_stability with
+          | Some user_stability -> Impl.format_stability user_stability |> String.uppercase
+          | None -> Q.AttrMap.get_no_ns FeedAttr.stability impl.Impl.props.Impl.attrs |> default "testing" in
 
-          impls |> List.iter (fun (impl, problem) ->
-            let from_feed = F.get_attr_ex FeedAttr.from_feed impl in
-            let impl_id = F.get_attr_ex FeedAttr.id impl in
-            let overrides = get_overrides from_feed in
-            let stability_value =
-              match StringMap.find impl_id overrides.F.user_stability with
-              | Some user_stability -> F.format_stability user_stability |> String.uppercase
-              | None -> Q.AttrMap.get_no_ns FeedAttr.stability impl.F.props.F.attrs |> default "testing" in
+        let arch_value =
+          match impl.Impl.os, impl.Impl.machine with
+          | None, None -> "any"
+          | os, machine -> Arch.format_arch os machine in
 
-            let arch_value =
-              match impl.F.os, impl.F.machine with
-              | None, None -> "any"
-              | os, machine -> Zeroinstall.Arch.format_arch os machine in
+        let notes_value =
+          match problem with
+          | None -> "None"
+          | Some problem -> Impl_provider.describe_problem impl problem in
 
-            let notes_value =
-              match problem with
-              | None -> "None"
-              | Some problem -> Zeroinstall.Impl_provider.describe_problem impl problem in
+        let (fetch_str, fetch_tip) = Gui.get_fetch_info config impl in
 
-            let (fetch_str, fetch_tip) = Zeroinstall.Gui.get_fetch_info config impl in
-
-            let row = model#append () in
-            model#set ~row ~column:version @@ Zeroinstall.Versions.format_version impl.F.parsed_version;
-            model#set ~row ~column:released @@ default "-" @@ Q.AttrMap.get_no_ns FeedAttr.released impl.F.props.F.attrs;
-            model#set ~row ~column:stability @@ stability_value;
-            model#set ~row ~column:langs @@ default "-" @@ Q.AttrMap.get_no_ns FeedAttr.langs impl.F.props.F.attrs;
-            model#set ~row ~column:arch arch_value;
-            model#set ~row ~column:notes notes_value;
-            model#set ~row ~column:weight (if Some impl = selected then 700 else 400);
-            model#set ~row ~column:unusable (problem <> None);
-            model#set ~row ~column:fetch fetch_str;
-            model#set ~row ~column:tooltip fetch_tip;
-            model#set ~row ~column:item impl;
-          )
+        let row = model#append () in
+        model#set ~row ~column:version @@ Version.to_string impl.Impl.parsed_version;
+        model#set ~row ~column:released @@ default "-" @@ Q.AttrMap.get_no_ns FeedAttr.released impl.Impl.props.Impl.attrs;
+        model#set ~row ~column:stability @@ stability_value;
+        model#set ~row ~column:langs @@ default "-" @@ Q.AttrMap.get_no_ns FeedAttr.langs impl.Impl.props.Impl.attrs;
+        model#set ~row ~column:arch arch_value;
+        model#set ~row ~column:notes notes_value;
+        model#set ~row ~column:weight (if Some impl = selected then 700 else 400);
+        model#set ~row ~column:unusable (problem <> None);
+        model#set ~row ~column:fetch fetch_str;
+        model#set ~row ~column:tooltip (Str.global_replace re_lt "&lt;" fetch_tip);
+        model#set ~row ~column:item impl;
+      )
   end
 
-let create tools ~trust_db reqs iface ~recalculate ~select_versions_tab ~watcher =
+let create tools ~trust_db reqs role ~recalculate ~select_versions_tab ~watcher =
   let config = tools#config in
-  let title = Printf.sprintf "Properties for %s" iface in
+  let title = Printf.sprintf "Properties for %s" (Solver.Model.Role.to_string role) in
   let dialog = GWindow.dialog ~title () in
   dialog#set_default_size
     ~width:(-1)
@@ -680,8 +683,8 @@ let create tools ~trust_db reqs iface ~recalculate ~select_versions_tab ~watcher
   (* Tabs *)
   let notebook = GPack.notebook ~packing:(dialog#vbox#pack ~expand:true ~fill:true) () in
   let label text = (GMisc.label ~text () :> GObj.widget) in
-  let feeds_tab = make_feeds_tab tools ~trust_db ~recalculate ~watcher dialog iface in
-  let versions_tab = make_versions_tab config reqs ~recalculate ~watcher dialog iface in
+  let feeds_tab = make_feeds_tab tools ~trust_db ~recalculate ~watcher dialog role.Solver.iface in
+  let versions_tab = make_versions_tab config reqs ~recalculate ~watcher dialog role in
   append_page notebook ~tab_label:(label "Feeds") (feeds_tab#widget);
   append_page notebook ~tab_label:(label "Versions") (versions_tab#widget);
 
@@ -702,7 +705,7 @@ let create tools ~trust_db reqs iface ~recalculate ~select_versions_tab ~watcher
   dialog#connect#response ==> (function
     | `COMPILE ->
         Gtk_utils.async ~parent:dialog (fun () ->
-          lwt () = Zeroinstall.Gui.compile config watcher#feed_provider iface ~autocompile:true in
+          lwt () = Gui.compile config watcher#feed_provider role.Solver.iface ~autocompile:true in
           recalculate ~force:false;
           Lwt.return ()
         )
@@ -713,7 +716,7 @@ let create tools ~trust_db reqs iface ~recalculate ~select_versions_tab ~watcher
   object
     method dialog = dialog
     method update : unit =
-      dialog#set_response_sensitive `COMPILE (Zeroinstall.Gui.have_source_for watcher#feed_provider iface);
+      dialog#set_response_sensitive `COMPILE (Gui.have_source_for watcher#feed_provider role.Solver.iface);
       feeds_tab#update;
       versions_tab#update
   end
