@@ -13,6 +13,7 @@ module Requirements = Zeroinstall.Requirements
 module U = Support.Utils
 module Progress = Zeroinstall.Progress
 module Downloader = Zeroinstall.Downloader
+module RoleMap = Zeroinstall.Solver.Model.RoleMap
 
 let main_window_help = Help_box.create "0install Help" [
 ("Overview",
@@ -141,13 +142,13 @@ let make_dialog opt_message mode ~systray =
 let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false) mode reqs ~refresh watcher : solver_box =
   let config = tools#config in
   let refresh = ref refresh in
-  let component_boxes = ref StringMap.empty in
+  let component_boxes = ref RoleMap.empty in
 
-  let report_bug iface =
+  let report_bug role =
     let run_test = test_callback |> pipe_some (fun test_callback ->
       Some (fun () -> Zeroinstall.Gui.run_test config tools#distro test_callback watcher#results)
     ) in
-    Bug_report_box.create ?run_test ?last_error:!Alert_box.last_error config ~iface ~results:watcher#results in
+    Bug_report_box.create ?run_test ?last_error:!Alert_box.last_error config ~role ~results:watcher#results in
 
   let need_recalculate = ref (Lwt.wait ()) in
   let recalculate ~force =
@@ -200,13 +201,13 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
     ) in
 
   (* Connect up the component tree view *)
-  let show_component iface ~select_versions_tab =
-    match StringMap.find iface !component_boxes with
+  let show_component role ~select_versions_tab =
+    match RoleMap.find role !component_boxes with
     | Some box -> box#dialog#present ()
     | None ->
-        let box = Component_box.create tools ~trust_db reqs iface ~recalculate ~select_versions_tab ~watcher in
-        component_boxes := !component_boxes |> StringMap.add iface box;
-        box#dialog#connect#destroy ==> (fun () -> component_boxes := !component_boxes |> StringMap.remove iface);
+        let box = Component_box.create tools ~trust_db reqs role ~recalculate ~select_versions_tab ~watcher in
+        component_boxes := !component_boxes |> RoleMap.add role box;
+        box#dialog#connect#destroy ==> (fun () -> component_boxes := !component_boxes |> RoleMap.remove role);
         box#update;
         box#dialog#show () in
 
@@ -233,7 +234,7 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
           match mode with
           | `Select_only -> on_success ()
           | `Download_only | `Select_for_run ->
-              let sels = results#get_selections in
+              let sels = Zeroinstall.Solver.selections results in
               match_lwt Driver.download_selections config tools#distro (lazy fetcher) ~include_packages:true ~feed_provider:watcher#feed_provider sels with
               | `aborted_by_user -> widgets.ok_button#set_active false; Lwt.return ()
               | `success -> on_success ()
@@ -286,7 +287,7 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
     | `ok ->
         let (ready, results) = watcher#results in
         assert ready;
-        `Success results#get_selections |> Lwt.return
+        `Success (Zeroinstall.Solver.selections results) |> Lwt.return
     | `aborted_by_user -> `Aborted_by_user |> Lwt.return in
 
   object
@@ -349,7 +350,7 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
       let (ready, _) = watcher#results in
       widgets.ok_button#misc#set_sensitive ready;
 
-      !component_boxes |> StringMap.iter (fun _iface box ->
+      !component_boxes |> RoleMap.iter (fun _role box ->
         box#update
       );
 
