@@ -5,8 +5,10 @@
 (* Testing the "0install" command-line interface *)
 
 open Zeroinstall.General
+open Zeroinstall
 open Support.Common
 open OUnit
+module Impl = Zeroinstall.Impl
 module Q = Support.Qdom
 module U = Support.Utils
 module F = Zeroinstall.Feed
@@ -29,8 +31,8 @@ exception Ok
 
 let handle_download_impls config pending_digests impls =
   impls |> List.iter (fun impl ->
-    match impl.F.impl_type with
-    | `cache_impl {F.digests;_} ->
+    match impl.Impl.impl_type with
+    | `cache_impl {Impl.digests;_} ->
         if Zeroinstall.Stores.lookup_maybe config.system digests config.stores = None then (
           let digest_str =
             digests |> U.first_match (fun digest ->
@@ -221,7 +223,7 @@ let suite = "0install">::: [
     fake_slave#allow_download "http://foo/Binary.xml" binary_feed;
     fake_slave#allow_download "http://foo/Source.xml" source_feed;
     let out = run ["update"; "http://foo/Binary.xml"; "--source"] in
-    assert_contains "Binary.xml: new -> 1.0" out;
+    assert_contains "Binary.xml#source: new -> 1.0" out;
     assert_contains "Compiler.xml: new -> 1.0" out;
 
     (* New compiler released. *)
@@ -255,9 +257,9 @@ let suite = "0install">::: [
 
     let local_uri = U.abspath system (feed_dir +/ "Local.xml") in
     let out = run ["download"; local_uri; "--xml"] in
-    let sels = selections_of_string out |> Zeroinstall.Selections.make_selection_map in
-    let sel = StringMap.find_safe local_uri sels in
-    assert_str_equal "0.1" @@ ZI.get_attribute "version" sel;
+    let sels = selections_of_string out in
+    let sel = Selections.(get_selected_ex {iface = local_uri; source = false}) sels in
+    assert_str_equal "0.1" @@ Element.version sel;
 
     let () =
       try ignore @@ run ["download"; "--offline"; (feed_dir +/ "selections.xml")]; assert false
@@ -668,9 +670,8 @@ let suite = "0install">::: [
 
     let out = run_0install fake_system ["select"; feed_dir +/ "Local.xml"; "--xml"] in
     let sels = selections_of_string ~path:local_uri out in
-    let index = Zeroinstall.Selections.make_selection_map sels in
-    let sel = StringMap.find_safe local_uri index in
-    assert_str_equal "0.1" (ZI.get_attribute "version" sel);
+    let sel = Selections.(get_selected_ex {iface = local_uri; source = false}) sels in
+    assert_str_equal "0.1" (Element.version sel);
 
     let out = run_0install fake_system ["select"; feed_dir +/ "runnable/RunExec.xml"] in
     assert_contains "Runner" out;
@@ -679,10 +680,9 @@ let suite = "0install">::: [
     fake_system#putenv "DISPLAY" ":foo";
     let out = run_0install fake_system ["select"; "--xml"; local_uri] in
     let sels = selections_of_string ~path:local_uri out in
-    let index = Zeroinstall.Selections.make_selection_map sels in
-    let sel = StringMap.find_safe local_uri index in
+    let sel = Selections.(get_selected_ex {iface = local_uri; source = false}) sels in
 
-    assert_str_equal "sha1=3ce644dc725f1d21cfcf02562c76f375944b266a" @@ ZI.get_attribute "id" sel;
+    assert_str_equal "sha1=3ce644dc725f1d21cfcf02562c76f375944b266a" @@ Element.id sel;
   );
 
   "config">:: Fake_system.with_fake_config (fun (config, fake_system) ->
