@@ -4,6 +4,7 @@
 
 (* These tests actually run a dummy web-server. *)
 
+open Zeroinstall
 open Zeroinstall.General
 open Support.Common
 open OUnit
@@ -19,12 +20,16 @@ let assert_contains = Fake_system.assert_contains
 let expect = Fake_system.expect
 let run_0install = Test_0install.run_0install
 
+let binary iface = {Selections.iface; source = false}
+
 exception Open_gui
 
 let parse_sels xml =
   try
-    let sels = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None |> Zeroinstall.Selections.create in
-    Zeroinstall.Selections.make_selection_map sels
+    `String (0, xml)
+    |> Xmlm.make_input
+    |> Q.parse_input None
+    |> Zeroinstall.Selections.create
   with Safe_exception _ as ex ->
     reraise_with_context ex "... parsing %s" xml
 
@@ -36,8 +41,7 @@ let get_sel_path config sel =
 
 let remove_cached config selections_path =
   let sels = Zeroinstall.Selections.load_selections config.system selections_path in
-  let index = Zeroinstall.Selections.make_selection_map sels in
-  let sel = StringMap.find_safe "http://example.com:8000/Hello.xml" index in
+  let sel = Zeroinstall.Selections.(get_selected_ex {iface = "http://example.com:8000/Hello.xml"; source = false}) sels in
   let stored = expect @@ get_sel_path config sel in
   assert (U.starts_with (Filename.basename stored) "sha1");
   U.rmtree ~even_if_locked:true config.system stored
@@ -66,8 +70,7 @@ let do_recipe config fake_system server ?(expected=[[("HelloWorld.tar.bz2", `Ser
   server#expect expected;
   let out = run_0install fake_system ["download"; feed; "--command="; "--xml"] in
   let sels = `String (0, out) |> Xmlm.make_input |> Q.parse_input None |> Zeroinstall.Selections.create in
-  let index = Zeroinstall.Selections.make_selection_map sels in
-  let sel = StringMap.find_safe feed index in
+  let sel = Zeroinstall.Selections.(get_selected_ex {iface = feed; source = false}) sels in
   get_sel_path config sel |> expect
 
 let suite = "download">::: [
@@ -332,7 +335,7 @@ let suite = "download">::: [
     ) in
     Fake_system.fake_log#assert_contains "Primary download failed; trying mirror URL 'http://roscidus.com/0mirror/archive/http%3A%23%23example\\(.\\|%2E\\)com%3A8000%23HelloWorld\\(.\\|%2E\\)tgz'";
     let sels = parse_sels out in
-    let sel = StringMap.find_safe "http://example.com:8000/Hello.xml" sels in
+    let sel = Zeroinstall.Selections.get_selected_ex (binary "http://example.com:8000/Hello.xml") sels in
     assert (fake_system#file_exists (expect (get_sel_path config sel) +/ "HelloWorld" +/ "main"))
   );
 
@@ -360,7 +363,7 @@ let suite = "download">::: [
     ) in
     Fake_system.fake_log#assert_contains ".* 404.*: trying implementation mirror at http://roscidus.com/0mirror";
     let sels = parse_sels out in
-    let sel = StringMap.find_safe "http://example.com:8000/Hello.xml" sels in
+    let sel = Zeroinstall.Selections.get_selected_ex (binary "http://example.com:8000/Hello.xml") sels in
     begin match Zeroinstall.Selections.get_source sel with
     | Zeroinstall.Selections.CacheSelection digests ->
         let path = Zeroinstall.Stores.lookup_any config.system digests config.stores in
@@ -433,8 +436,7 @@ let suite = "download">::: [
       [("HelloWorld.tgz", `Serve)];
     ];
 
-    let index = Zeroinstall.Selections.make_selection_map sels in
-    let sel = StringMap.find_safe "http://example.com:8000/Hello.xml" index in
+    let sel = Zeroinstall.Selections.get_selected_ex (binary "http://example.com:8000/Hello.xml") sels in
     assert_equal None @@ get_sel_path config sel;
 
     let out = run_0install fake_system ["download"; Test_0install.feed_dir +/ "selections.xml"] in
@@ -564,9 +566,8 @@ let suite = "download">::: [
     assert (1.0 <> get_mtime selections_path);
 
     let sels = Zeroinstall.Selections.load_selections system selections_path in
-    let index = Zeroinstall.Selections.make_selection_map sels in
-    let sel = StringMap.find_safe "http://example.com:8000/Hello.xml" index in
-    assert_equal "sha1=3ce644dc725f1d21cfcf02562c76f375944b266a" (ZI.get_attribute "id" sel);
+    let sel = Zeroinstall.Selections.get_selected_ex (binary "http://example.com:8000/Hello.xml") sels in
+    assert_equal "sha1=3ce644dc725f1d21cfcf02562c76f375944b266a" (Element.id sel);
 
     (* Untrust the key - we'll need to use the GUI to confirm it again *)
     trust_db#untrust_key ~domain "DE937DD411906ACF7C263B396FCF121BE2390E0B";
