@@ -13,7 +13,7 @@ module F = Zeroinstall.Feed
 module B = Zeroinstall.Binding
 
 let feed_of_xml system xml =
-  let root = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None in
+  let root = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None |> Element.parse_feed in
   F.parse system root None
 
 let suite = "feed">::: [
@@ -21,7 +21,7 @@ let suite = "feed">::: [
     let (_config, fake_system) = Fake_system.get_fake_config None in
     let system = (fake_system :> system) in
     let local_path = Test_0install.feed_dir +/ "Local.xml" in
-    let root = Q.parse_file system local_path in
+    let root = Q.parse_file system local_path |> Element.parse_feed in
     let feed = F.parse system root (Some local_path) in
 
     let () =
@@ -70,12 +70,12 @@ let suite = "feed">::: [
 
   "command">:: Fake_system.with_fake_config (fun (config, _fake_system) ->
     let path = Test_0install.feed_dir +/ "Command.xml" in
-    let root = Q.parse_file config.system path in
+    let root = Q.parse_file config.system path |> Element.parse_feed in
     let feed = F.parse config.system root (Some path) in
 
     let path name impl =
-      let command = StringMap.find_safe name impl.F.props.F.commands in
-      ZI.get_attribute "path" command.F.command_qdom in
+      let command = StringMap.find_safe name impl.Impl.props.Impl.commands in
+      ZI.get_attribute "path" command.Impl.command_qdom in
 
     let a = StringMap.find_safe "a" feed.F.implementations in
     Fake_system.assert_str_equal "foo" @@ path "run" a;
@@ -105,13 +105,13 @@ let suite = "feed">::: [
           <implementation id='sha1=345' version='2'/>\n\
         </group>\n\
       </interface>" in
-    let root = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None in
+    let root = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None |> Element.parse_feed in
     let feed = F.parse config.system root (Some "/local.xml") in
     begin match StringMap.bindings feed.F.implementations with
     | [("sha1=124", s124); ("sha1=234", s234); ("sha1=345", s345)] ->
-        assert_equal [("fr", None)] @@ F.get_langs s124;
-        assert_equal [("fr", None); ("en", Some "gb")] @@ F.get_langs s234;
-        assert_equal [("en", None)] @@ F.get_langs s345;
+        assert_equal [("fr", None)] @@ Impl.get_langs s124;
+        assert_equal [("fr", None); ("en", Some "gb")] @@ Impl.get_langs s234;
+        assert_equal [("en", None)] @@ Impl.get_langs s345;
     | _ -> assert false end;
 
     begin match feed.F.imported_feeds with
@@ -138,18 +138,18 @@ let suite = "feed">::: [
          </implementation>\n\
         </group>\n\
       </interface>" in
-    let root = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None in
+    let root = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None |> Element.parse_feed in
     let feed = F.parse config.system root (Some "/local.xml") in
 
     begin match StringMap.bindings feed.F.implementations with
     | [("sha1=123", impl)] ->
-        begin match impl.F.props.F.bindings |> List.map B.parse_binding with
-        | [Some B.EnvironmentBinding {B.mode = B.Replace; _}] -> ()
+        begin match impl.Impl.props.Impl.bindings |> List.map Element.classify_binding |> List.map B.parse_binding with
+        | [B.EnvironmentBinding {B.mode = B.Replace; _}] -> ()
         | _ -> assert false end;
 
-        begin match impl.F.props.F.requires with
+        begin match impl.Impl.props.Impl.requires with
         | [dep] ->
-            begin match dep.F.dep_qdom |> ZI.filter_map B.parse_binding with
+            begin match Element.bindings dep.Impl.dep_qdom |> List.map B.parse_binding with
             [
               B.EnvironmentBinding ({ B.mode = B.Add {B.pos = B.Prepend; _ }; _ } as b0);
               B.EnvironmentBinding ({ B.mode = B.Add {B.pos = B.Prepend; B.default = None; _ }; _ });
@@ -163,13 +163,12 @@ let suite = "feed">::: [
               );
 
               let env = Env.create [| |] in
-              let impls = StringMap.singleton "http://example.com/" ((), Some "/impl") in
 
               let check ?old binding =
                 begin match old with
                 | None -> Env.unset env "PATH"
                 | Some old -> Env.put env "PATH" old end;
-                B.do_env_binding env impls "http://example.com/" binding;
+                B.do_env_binding env (lazy ((), Some "/impl")) binding;
                 Env.get_exn env "PATH" in
 
               Fake_system.assert_str_equal "/impl/bin:/bin:/usr/bin" @@ check b0;
@@ -189,9 +188,8 @@ let suite = "feed">::: [
 
     let check ?impl ?old binding =
       let env = Env.create [| |] in
-      let impls = StringMap.singleton "http://example.com/" ((), impl) in
       old |> if_some (Env.put env binding.B.var_name);
-      B.do_env_binding env impls "http://example.com/" binding;
+      B.do_env_binding env (lazy ((), impl)) binding;
       Env.get_exn env binding.B.var_name in
 
     Fake_system.assert_str_equal "/impl/lib:/usr/lib" @@ check prepend ~impl:"/impl" ~old:"/usr/lib";
@@ -236,15 +234,15 @@ let suite = "feed">::: [
 
     match feed.F.implementations |> StringMap.bindings with
     | [("sha1=123", impl)] ->
-        begin match impl.F.props.F.requires with
+        begin match impl.Impl.props.Impl.requires with
         | [dep; dep2] ->
-            begin match dep.F.dep_restrictions with
+            begin match dep.Impl.dep_restrictions with
             | [res] -> Fake_system.assert_str_equal "version 2.3.4..!3.4.5" @@ res#to_string;
             | _ -> assert false end;
-            assert_equal [] dep2.F.dep_restrictions;
+            assert_equal [] dep2.Impl.dep_restrictions;
 
-            dep.F.dep_qdom.Q.attrs |> Q.AttrMap.get ("http://my/namespace", "foo") |> assert_equal (Some "test");
-            dep.F.dep_qdom.Q.attrs |> Q.AttrMap.get ("http://my/namespace", "food") |> assert_equal None;
+            (Element.as_xml dep.Impl.dep_qdom).Q.attrs |> Q.AttrMap.get ("http://my/namespace", "foo") |> assert_equal (Some "test");
+            (Element.as_xml dep.Impl.dep_qdom).Q.attrs |> Q.AttrMap.get ("http://my/namespace", "food") |> assert_equal None;
         | _ -> assert false end;
     | _ -> assert false
   );
@@ -265,8 +263,8 @@ let suite = "feed">::: [
 
     match feed.F.implementations |> StringMap.bindings with
     | [("sha1=123", impl); ("used", used)] ->
-        Fake_system.assert_str_equal "1.0-rc3-pre" @@ Zeroinstall.Versions.format_version impl.F.parsed_version;
-        Fake_system.assert_str_equal "2" @@ Zeroinstall.Versions.format_version used.F.parsed_version;
+        Fake_system.assert_str_equal "1.0-rc3-pre" @@ Zeroinstall.Version.to_string impl.Impl.parsed_version;
+        Fake_system.assert_str_equal "2" @@ Zeroinstall.Version.to_string used.Impl.parsed_version;
     | _ -> assert false
   );
 
@@ -288,7 +286,7 @@ let suite = "feed">::: [
     match feed.F.implementations |> StringMap.bindings with
     | [("sha1=123", impl1); ("sha1=124", impl2)] ->
         let check expected name impl =
-          let attr = Q.AttrMap.get name impl.F.props.F.attrs |> default "" in
+          let attr = Q.AttrMap.get name impl.Impl.props.Impl.attrs |> default "" in
           Fake_system.assert_str_equal expected attr in
 
         check "foovalue" ("", "foo") impl1;
@@ -343,7 +341,7 @@ let suite = "feed">::: [
   "replaced">:: (fun () ->
     let system = (new Fake_system.fake_system None :> system) in
     let iface = Test_0install.feed_dir +/ "Replaced.xml" in
-    let root = Q.parse_file system iface in
+    let root = Q.parse_file system iface |> Element.parse_feed in
     let feed = F.parse system root (Some iface) in
     Fake_system.assert_str_equal "http://localhost:8000/Hello" (Fake_system.expect feed.F.replacement)
   );
