@@ -16,10 +16,10 @@ module Q = Support.Qdom
 let get_matching_package_impls distro feed =
   let best_score = ref 0 in
   let best_impls = ref [] in
-  ListLabels.iter feed.Feed.package_implementations ~f:(function (elem, _) as package_impl ->
-    match ZI.get_attribute_opt FeedAttr.package elem with
-    | Some package_name when distro#is_valid_package_name package_name ->
-        let distributions = default "" @@ ZI.get_attribute_opt "distributions" elem in
+  feed.Feed.package_implementations |> List.iter (function (elem, _) as package_impl ->
+    let package_name = Element.package elem in
+    if distro#is_valid_package_name package_name then (
+        let distributions = default "" @@ Element.distributions elem in
         let distro_names = Str.split_delim U.re_space distributions in
         let score_this_item =
           if distro_names = [] then 1                                 (* Generic <package-implementation>; no distribution specified *)
@@ -32,21 +32,21 @@ let get_matching_package_impls distro feed =
         if score_this_item = !best_score then (
           best_impls := package_impl :: !best_impls
         )
-    | _ -> ()
+    )
   );
   !best_impls
 
 type query = {
-  elem : Support.Qdom.element;      (* The <package-element> which generated this query *)
+  elem : [`package_impl] Element.t; (* The <package-element> which generated this query *)
   package_name : string;            (* The 'package' attribute on the <package-element> *)
-  elem_props : Feed.properties;     (* Properties on or inherited by the <package-element> - used by [add_package_implementation] *)
+  elem_props : Impl.properties;     (* Properties on or inherited by the <package-element> - used by [add_package_implementation] *)
   feed : Feed.feed;                 (* The feed containing the <package-element> *)
-  results : Feed.distro_implementation Support.Common.StringMap.t ref;
+  results : Impl.distro_implementation Support.Common.StringMap.t ref;
 }
 
 let make_query feed elem elem_props results = {
   elem;
-  package_name = ZI.get_attribute Constants.FeedAttr.package elem;
+  package_name = Element.package elem;
   elem_props;
   feed;
   results;
@@ -84,13 +84,12 @@ let get_quick_test_attrs path =
   |> Q.AttrMap.add_no_ns FeedAttr.quick_test_file path
   |> Q.AttrMap.add_no_ns FeedAttr.quick_test_mtime (Printf.sprintf "%.0f" mtime)
 
-let make_restricts_distro iface_uri distros =
-  let elem = ZI.make "restricts" in
-  let open Feed in {
-    dep_qdom = elem;
-    dep_importance = Dep_restricts;
+let make_restricts_distro iface_uri distros = { Impl.
+    dep_qdom = Element.dummy_restricts;
+    dep_importance = `restricts;
     dep_iface = iface_uri;
-    dep_restrictions = [make_distribtion_restriction distros];
+    dep_src = false;
+    dep_restrictions = [Impl.make_distribtion_restriction distros];
     dep_required_commands = [];
     dep_if_os = None;
     dep_use = None;
@@ -118,26 +117,26 @@ class virtual distribution config =
     )
   ) in
 
-  let make_host_impl path version ?(commands=StringMap.empty) ?(requires=[]) from_feed id =
+  let make_host_impl path version ~package ?(commands=StringMap.empty) ?(requires=[]) from_feed id =
     let host_machine = system#platform in
-    let open Feed in
-    let props = {
+    let props = { Impl.
       attrs = get_quick_test_attrs path
         |> Q.AttrMap.add_no_ns FeedAttr.from_feed (Feed_url.format_url (`distribution_feed from_feed))
         |> Q.AttrMap.add_no_ns FeedAttr.id id
         |> Q.AttrMap.add_no_ns FeedAttr.stability "packaged"
-        |> Q.AttrMap.add_no_ns FeedAttr.version version;
+        |> Q.AttrMap.add_no_ns FeedAttr.version version
+        |> Q.AttrMap.add_no_ns FeedAttr.package package;
       requires;
       bindings = [];
       commands;
-    } in {
+    } in { Impl.
       qdom = ZI.make "host-package-implementation";
       props;
       stability = Packaged;
       os = None;
       machine = Some host_machine.Platform.machine;       (* (hopefully) *)
-      parsed_version = Versions.parse_version version;
-      impl_type = `package_impl {
+      parsed_version = Version.parse version;
+      impl_type = `package_impl { Impl.
         package_distro = "host";
         package_state = `installed;
       }
@@ -155,21 +154,21 @@ class virtual distribution config =
               Q.AttrMap.singleton "name" "run"
               |> Q.AttrMap.add_no_ns "path" path
             ) in
-          let commands = StringMap.singleton "run" Feed.({command_qdom = run; command_requires = []; command_bindings = []}) in
-          (id, make_host_impl path version ~commands url id)
+          let commands = StringMap.singleton "run" Impl.({command_qdom = run; command_requires = []; command_bindings = []}) in
+          (id, make_host_impl ~package:"host-python" path version ~commands url id)
         )
     | `remote_feed "http://repo.roscidus.com/python/python-gobject" as url ->
         Lazy.force python_info |> U.filter_map (function
           | (_, Some (path, version)) ->
               let id = "package:host:python-gobject:" ^ version in
               let requires = [make_restricts_distro "http://repo.roscidus.com/python/python" "host"] in
-              Some (id, make_host_impl path version ~requires url id)
+              Some (id, make_host_impl ~package:"host-python-gobject" path version ~requires url id)
           | (_, None) -> None
         )
     | _ -> [] in
 
   let fixup_main distro_get_correct_main impl =
-    let open Feed in
+    let open Impl in
     match get_command_opt "run" impl with
     | None -> ()
     | Some run ->
@@ -204,7 +203,7 @@ class virtual distribution config =
 
     (** Convenience wrapper for [add_result] that builds a new implementation from the given attributes. *)
     method private add_package_implementation ?id ?main (query:query) ~version ~machine ~quick_test ~package_state ~distro_name =
-      let version_str = Versions.format_version version in
+      let version_str = Version.to_string version in
       let id = id |? lazy (Printf.sprintf "%s:%s:%s:%s" id_prefix query.package_name version_str (default "*" machine)) in
       let props = query.elem_props in
       let elem = query.elem in
@@ -214,7 +213,7 @@ class virtual distribution config =
         | None -> props
         | Some path ->
             (* We may add or modify the main executable path. *)
-            let open Feed in
+            let open Impl in
             let run_command =
               match StringMap.find "run" props.commands with
               | Some command ->
@@ -227,7 +226,7 @@ class virtual distribution config =
                   make_command "run" path in
             {props with commands = StringMap.add "run" run_command props.commands} in
 
-      let new_attrs = ref props.Feed.attrs in
+      let new_attrs = ref props.Impl.attrs in
       let set name value =
         new_attrs := Q.AttrMap.add_no_ns name value !new_attrs in
       set "id" id;
@@ -243,9 +242,9 @@ class virtual distribution config =
           | UnchangedSince mtime ->
               set FeedAttr.quick_test_mtime (Int64.of_float mtime |> Int64.to_string) end;
 
-      let open Feed in
+      let open Impl in
       let impl = {
-        qdom = elem;
+        qdom = Element.as_xml elem;
         os = None;
         machine;
         stability = Packaged;
@@ -263,8 +262,8 @@ class virtual distribution config =
      * returned map. Override this if you can provide a more efficient implementation. *)
     method is_installed elem =
       let master_feed =
-        match ZI.get_attribute_opt FeedAttr.from_feed elem with
-        | None -> ZI.get_attribute FeedAttr.interface elem |> Feed_url.parse_non_distro (* (for very old selections documents) *)
+        match Element.from_feed elem with
+        | None -> Element.interface elem |> Feed_url.parse_non_distro (* (for very old selections documents) *)
         | Some from_feed ->
             match Feed_url.parse from_feed with
             | `distribution_feed master_feed -> master_feed
@@ -272,15 +271,15 @@ class virtual distribution config =
       match Feed_cache.get_cached_feed config master_feed with
       | None -> false
       | Some master_feed ->
-          let wanted_id = ZI.get_attribute FeedAttr.id elem in
+          let wanted_id = Element.id elem in
           let impls = self#get_impls_for_feed master_feed in
           match StringMap.find wanted_id impls with
           | None -> false
-          | Some {Feed.impl_type = `package_impl {Feed.package_state; _}; _} -> package_state = `installed
+          | Some {Impl.impl_type = `package_impl {Impl.package_state; _}; _} -> package_state = `installed
 
     (** Get the native implementations (installed or candidates for installation) for this feed.
      * This default implementation finds the best <package-implementation> elements and calls [get_package_impls] on each one. *)
-    method get_impls_for_feed ?(init=StringMap.empty) (feed:Feed.feed) : Feed.distro_implementation StringMap.t =
+    method get_impls_for_feed ?(init=StringMap.empty) (feed:Feed.feed) : Impl.distro_implementation StringMap.t =
       let results = ref init in
 
       if check_host_python then (
@@ -315,7 +314,7 @@ class virtual distribution config =
         The default implementation checks that main exists, and searches [system_paths] for
         it if not. *)
     method private get_correct_main _impl run_command =
-      let open Feed in
+      let open Impl in
       ZI.get_attribute_opt "path" run_command.command_qdom |> pipe_some (fun path ->
         if Filename.is_relative path || not (system#file_exists path) then (
           (* Need to search for the binary *)
@@ -342,7 +341,7 @@ class virtual distribution config =
       | matches ->
           lwt available = packagekit#is_available in
           if available then (
-            let package_names = matches |> List.map (fun (elem, _props) -> ZI.get_attribute "package" elem) in
+            let package_names = matches |> List.map (fun (elem, _props) -> Element.package elem) in
             let hint = Feed_url.format_url feed.Feed.url in
             packagekit#check_for_candidates ~ui ~hint package_names
           ) else Lwt.return ()
@@ -359,7 +358,7 @@ class virtual distribution config =
                 );
                 Lwt.return `ok end
         | _ ->
-            let names = items |> List.map (fun (_impl, rm) -> snd rm.Feed.distro_install_info) in
+            let names = items |> List.map (fun (_impl, rm) -> snd rm.Impl.distro_install_info) in
             ui#confirm (Printf.sprintf
               "This program depends on some packages that are available through your distribution. \
                Please install them manually using %s and try again. Or, install 'packagekit' and I can \
@@ -367,26 +366,26 @@ class virtual distribution config =
   end
 
 let is_installed config (distro:distribution) elem =
-  match ZI.get_attribute_opt FeedAttr.quick_test_file elem with
+  match Element.quick_test_file elem with
   | None ->
-      let package_name = ZI.get_attribute FeedAttr.package elem in
+      let package_name = Element.package elem in
       distro#is_valid_package_name package_name && distro#is_installed elem
   | Some file ->
       match config.system#stat file with
       | None -> false
       | Some info ->
-          match ZI.get_attribute_opt FeedAttr.quick_test_mtime elem with
+          match Element.quick_test_mtime elem with
           | None -> true      (* quick-test-file exists and we don't care about the time *)
-          | Some required_mtime -> (Int64.of_float info.Unix.st_mtime) = Int64.of_string required_mtime
+          | Some required_mtime -> (Int64.of_float info.Unix.st_mtime) = required_mtime
 
 let install_distro_packages (distro:distribution) ui impls : [ `ok | `cancel ] Lwt.t =
   let groups = ref StringMap.empty in
   impls |> List.iter (fun impl ->
-    let `package_impl {Feed.package_state; _} = impl.Feed.impl_type in
+    let `package_impl {Impl.package_state; _} = impl.Impl.impl_type in
     match package_state with
-    | `installed -> raise_safe "BUG: package %s already installed!" (Feed.get_id impl).Feed_url.id
+    | `installed -> raise_safe "BUG: package %s already installed!" (Impl.get_id impl).Feed_url.id
     | `uninstalled rm ->
-        let (typ, _info) = rm.Feed.distro_install_info in
+        let (typ, _info) = rm.Impl.distro_install_info in
         let items = default [] @@ StringMap.find typ !groups in
         groups := StringMap.add typ ((impl, rm) :: items) !groups
   );
