@@ -5,11 +5,9 @@
 (** Interacting with distribution package managers. *)
 
 open General
-open Support
 open Support.Common
 module FeedAttr = Constants.FeedAttr
 module U = Support.Utils
-module Q = Support.Qdom
 
 open Distro
 
@@ -22,7 +20,7 @@ let generic_distribution config =
   end
 
 let try_cleanup_distro_version_warn version package_name =
-  match Versions.try_cleanup_distro_version version with
+  match Version.try_cleanup_distro_version version with
   | None -> log_warning "Can't parse distribution version '%s' for package '%s'" version package_name; None
   | Some zi_version -> Some zi_version
 
@@ -36,17 +34,13 @@ let iter_dir system fn path =
     matches the [id] attribute on [elem].
     Returns [false] if the cache is out-of-date. *)
 let check_cache id_prefix elem (cache:Distro_cache.cache) =
-  match ZI.get_attribute_opt "package" elem with
-  | None ->
-      Qdom.log_elem Support.Logging.Warning "Missing 'package' attribute" elem;
-      false
-  | Some package ->
-      let sel_id = ZI.get_attribute "id" elem in
-      let matches (installed_version, machine) =
-        let installed_id = Printf.sprintf "%s:%s:%s:%s" id_prefix package (Versions.format_version installed_version) (default "*" machine) in
-        (* log_warning "Want %s %s, have %s" package sel_id installed_id; *)
-        sel_id = installed_id in
-      List.exists matches (fst (cache#get package))
+  let package = Element.package elem in
+  let sel_id = Element.id elem in
+  let matches (installed_version, machine) =
+    let installed_id = Printf.sprintf "%s:%s:%a:%s" id_prefix package Version.fmt installed_version (default "*" machine) in
+    (* log_warning "Want %s %s, have %s" package sel_id installed_id; *)
+    sel_id = installed_id in
+  List.exists matches (fst (cache#get package))
 
 module Debian = struct
   let dpkg_db_status = "/var/lib/dpkg/status"
@@ -58,7 +52,7 @@ module Debian = struct
     | status -> Support.System.check_exit_status status
 
   type apt_cache_entry = {
-    version : string;
+    version : Version.t;
     machine : string;
     size : Int64.t option;
   }
@@ -92,7 +86,7 @@ module Debian = struct
               done
             with Stream.Failure -> () end;
             match !version, !machine with
-            | Some version, Some machine -> Lwt.return (Some {version = Versions.format_version version; machine; size = !size})
+            | Some version, Some machine -> Lwt.return (Some {version; machine; size = !size})
             | _ -> Lwt.return None
           with ex ->
             log_warning ~ex "'apt-cache show %s' failed" package;
@@ -133,7 +127,7 @@ module Debian = struct
       !results in
 
     let fixup_java_main impl java_version =
-      let java_arch = if impl.Feed.machine = Some "x86_64" then Some "amd64" else impl.Feed.machine in
+      let java_arch = if impl.Impl.machine = Some "x86_64" then Some "amd64" else impl.Impl.machine in
 
       match java_arch with
       | None -> log_warning "BUG: Missing machine type on Java!"; None
@@ -175,9 +169,8 @@ module Debian = struct
         if use_apt_cache_results then (
           let entry = try Hashtbl.find apt_cache package_name with Not_found -> None in
           entry |> if_some (fun {version; machine; size = _} ->
-            let version = Versions.parse_version version in
             let machine = Arch.none_if_star machine in
-            let package_state = `uninstalled Feed.({distro_size = None; distro_install_info = ("apt-get install", package_name)}) in
+            let package_state = `uninstalled Impl.({distro_size = None; distro_install_info = ("apt-get install", package_name)}) in
             self#add_package_implementation ~package_state ~version ~machine ~quick_test:None ~distro_name query
           )
         );
@@ -192,7 +185,7 @@ module Debian = struct
         match Distro.get_matching_package_impls self feed with
         | [] -> Lwt.return ()
         | matches ->
-            let package_names = matches |> List.map (fun (elem, _props) -> (ZI.get_attribute "package" elem)) in
+            let package_names = matches |> List.map (fun (elem, _props) -> (Element.package elem)) in
 
             (* Check apt-cache to see whether we have the pacakges. If PackageKit isn't available, we'll use these
              * results directly. If it is available, we'll use these results to filter the PackageKit query, because
@@ -214,7 +207,7 @@ module Debian = struct
       method! private add_package_implementation ?id ?main query ~version ~machine ~quick_test ~package_state ~distro_name =
         let version =
           match query.package_name, version with
-          | ("openjdk-6-jre" | "openjdk-7-jre"), (([major], Versions.Pre) :: (minor, mmod) :: rest) ->
+          | ("openjdk-6-jre" | "openjdk-7-jre"), (([major], Version.Pre) :: (minor, mmod) :: rest) ->
             (* Debian marks all Java versions as pre-releases
                See: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=685276 *)
             (major :: minor, mmod) :: rest
@@ -223,7 +216,7 @@ module Debian = struct
         super#add_package_implementation ?id ?main query ~version ~machine ~quick_test ~package_state ~distro_name
 
       method! private get_correct_main impl run_command =
-        let id = Feed.get_attr_ex Constants.FeedAttr.id impl in
+        let id = Impl.get_attr_ex Constants.FeedAttr.id impl in
         if U.starts_with id "package:deb:openjdk-6-jre:" then
           fixup_java_main impl "6-openjdk"
         else if U.starts_with id "package:deb:openjdk-7-jre:" then
@@ -240,7 +233,7 @@ module RPM = struct
     let fixup_java_main impl java_version =
       (* (note: on Fedora, unlike Debian, the arch is x86_64, not amd64) *)
 
-      match impl.Feed.machine with
+      match impl.Impl.machine with
       | None -> log_warning "BUG: Missing machine type on Java!"; None
       | Some java_arch ->
           let java_bin = Printf.sprintf "/usr/lib/jvm/jre-%s.%s/bin/java" java_version java_arch in
@@ -298,7 +291,7 @@ module RPM = struct
 
       method! private get_correct_main impl run_command =
         (* OpenSUSE uses _, Fedora uses . *)
-        let id = Feed.get_attr_ex Constants.FeedAttr.id impl in
+        let id = Impl.get_attr_ex Constants.FeedAttr.id impl in
         let starts x = U.starts_with id x in
         if starts "package:rpm:java-1.6.0-openjdk:" || starts "package:rpm:java-1_6_0-openjdk:" then
           fixup_java_main impl "1.6.0-openjdk"
@@ -310,10 +303,7 @@ module RPM = struct
       method! private add_package_implementation ?id ?main query ~version ~machine ~quick_test ~package_state ~distro_name =
         let version =
           (* OpenSUSE uses _, Fedora uses . *)
-          let package_name = String.copy query.package_name in
-          for i = 0 to String.length package_name - 1 do
-            if package_name.[i] = '_' then package_name.[i] <- '.'
-          done;
+          let package_name = query.package_name |> String.map (function '_' -> '.' | x -> x) in
           match package_name with
           | "java-1.6.0-openjdk" | "java-1.7.0-openjdk"
           | "java-1.6.0-openjdk-devel" | "java-1.7.0-openjdk-devel" ->
@@ -465,7 +455,7 @@ module Mac = struct
               self#add_package_implementation
                 ~main
                 ~package_state:`installed
-                ~version:(Versions.parse_version zero_version)
+                ~version:(Version.parse zero_version)
                 ~machine:(Some machine)
                 ~quick_test:(Some (main, UnchangedSince info.Unix.st_mtime))
                 ~distro_name
@@ -601,10 +591,10 @@ module Win = struct
         [(netfx32_install, "i486"); (netfx64_install, "x86_64")] |> List.iter (function
           | None, _ -> ()
           | Some install, machine ->
-              let version = Versions.parse_version zero_version in
+              let version = Version.parse zero_version in
               let package_state =
                 if install = 1 then `installed
-                else `uninstalled Feed.({distro_size = None; distro_install_info = ("Windows installer", "NetFX")}) in
+                else `uninstalled Impl.({distro_size = None; distro_install_info = ("Windows installer", "NetFX")}) in
               self#add_package_implementation
                 ~main:""      (* .NET executables do not need a runner on Windows but they need one elsewhere *)
                 ~package_state
@@ -622,10 +612,10 @@ module Win = struct
 
         [(netfx32_install, netfx32_release, "i486"); (netfx64_install, netfx64_release, "x86_64")] |> List.iter (function
           | Some install, Some release, machine ->
-              let version = Versions.parse_version zero_version in
+              let version = Version.parse zero_version in
               let package_state =
                 if install = 1 && release >= release_version then `installed
-                else `uninstalled Feed.({distro_size = None; distro_install_info = ("Windows installer", "NetFX")}) in
+                else `uninstalled Impl.({distro_size = None; distro_install_info = ("Windows installer", "NetFX")}) in
               self#add_package_implementation
                 ~main:""      (* .NET executables do not need a runner on Windows but they need one elsewhere *)
                 ~package_state
@@ -648,7 +638,7 @@ module Win = struct
               match config.system#stat java_bin with
               | None -> ()
               | Some info ->
-                  let version = Versions.parse_version zero_version in
+                  let version = Version.parse zero_version in
                   let quick_test = Some (java_bin, UnchangedSince info.Unix.st_mtime) in
                   self#add_package_implementation
                     ~main:java_bin
