@@ -8,7 +8,7 @@ class type feed_provider =
   object
     method forget_distro : Feed_url.non_distro_feed -> unit
     method forget_user_feeds : General.iface_uri -> unit
-    method get_distro_impls : Feed.feed -> Feed.distro_implementation Support.Common.StringMap.t * Feed.feed_overrides
+    method get_distro_impls : Feed.feed -> Impl.distro_implementation Support.Common.StringMap.t * Feed.feed_overrides
     method get_feed : Feed_url.non_distro_feed -> (Feed.feed * Feed.feed_overrides) option
     method get_feeds_used : Feed_url.non_distro_feed list
     method get_iface_config : General.iface_uri -> Feed_cache.interface_config
