@@ -10,10 +10,10 @@ module Q = Support.Qdom
 module G = Support.Gpg
 module FeedAttr = Constants.FeedAttr
 
-type non_mirror_case = [ `ok of Q.element | `no_trusted_keys | `replay_attack of (feed_url * float * float) | `aborted_by_user ]
+type non_mirror_case = [ `ok of [`feed] Element.t | `no_trusted_keys | `replay_attack of (feed_url * float * float) | `aborted_by_user ]
 
 type fetch_feed_response =
-  [ `update of (Q.element * fetch_feed_response Lwt.t option)  (* Use this version (but a better version may come soon) *)
+  [ `update of ([`feed] Element.t * fetch_feed_response Lwt.t option)  (* Use this version (but a better version may come soon) *)
   | `aborted_by_user        (* Abort silently (no need to notify the user) *)
   | `problem of (string * fetch_feed_response Lwt.t option)    (* Report a problem (but may still succeed later) *)
   | `no_update ]            (* Use the previous version *)
@@ -122,11 +122,14 @@ class fetcher config (trust_db:Trust.trust_db) (distro:Distro.distribution) (dow
       ) in
 
       if have_valid then (
-        errors |> List.iter (fun ex -> log_warning ~ex "Error downloading key for %s" feed);
+        errors |> List.iter (fun ex -> log_warning ~ex "Error checking signature for %s" feed);
         `success (sigs, messages) |> Lwt.return
       ) else (
         let msg = errors |> List.map Printexc.to_string |> String.concat ";" in
-        `problem (Printf.sprintf "Error downloading key for '%s': %s" feed msg) |> Lwt.return
+        let msg =
+          if messages <> "" then msg ^ "\n" ^ messages
+          else msg in
+        `problem (Printf.sprintf "Error checking signature for '%s': %s" feed msg) |> Lwt.return
       )
     ) in
 
@@ -138,12 +141,12 @@ class fetcher config (trust_db:Trust.trust_db) (distro:Distro.distribution) (dow
     log_debug "Updating '%s' from network; modified at %s" feed_url pretty_time;
 
     (* Check the new XML is valid before adding it *)
-    let new_root = `String (0, new_xml) |> Xmlm.make_input |> Q.parse_input (Some feed_url) in
+    let new_root = `String (0, new_xml) |> Xmlm.make_input |> Q.parse_input (Some feed_url) |> Element.parse_feed in
     ignore (Feed.parse system new_root None).Feed.root;
 
-    let url_in_feed = ZI.get_attribute FeedAttr.uri new_root in
+    let url_in_feed = Element.uri_exn new_root in
     if url_in_feed <> feed_url then
-      Q.raise_elem "URL mismatch in feed:\n%s expected\n%s given in 'uri' attribute on" feed_url url_in_feed new_root;
+      raise_safe "URL mismatch in feed:\n%s expected\n%s given in 'uri' attribute on %a" feed_url url_in_feed Element.fmt new_root;
 
     (* Load the old XML *)
     let cache_path = Feed_cache.get_save_cache_path config feed in
@@ -410,7 +413,7 @@ class fetcher config (trust_db:Trust.trust_db) (distro:Distro.distribution) (dow
         resolved in
 
     try_lwt
-      let {Feed_url.feed; Feed_url.id = _} = Feed.get_id impl in
+      let {Feed_url.feed; Feed_url.id = _} = Impl.get_id impl in
       let open Recipe in
       (* Start all the downloads. The downloads happen in parallel, each returning
        * a future that will perform the extraction step. These futures are evaluated in sequence. *)
@@ -507,8 +510,8 @@ class fetcher config (trust_db:Trust.trust_db) (distro:Distro.distribution) (dow
               log_info "Error from mirror: %s" mirror_msg;
               raise_safe "%s" orig_msg
     with Safe_exception _ as ex ->
-      let {Feed_url.feed; Feed_url.id} = Feed.get_id impl in
-      let version = Feed.get_attr_ex FeedAttr.version impl in
+      let {Feed_url.feed; Feed_url.id} = Impl.get_id impl in
+      let version = Impl.get_attr_ex FeedAttr.version impl in
       reraise_with_context ex "... downloading implementation %s %s (id=%s)" (Feed_url.format_url feed) version id in
 
   object
@@ -582,34 +585,34 @@ class fetcher config (trust_db:Trust.trust_db) (distro:Distro.distribution) (dow
                       `problem (msg, Some (wait_for_mirror mirror)) |> Lwt.return
               )
 
-    method download_impls (impls:Feed.generic_implementation list) : [ `success | `aborted_by_user ] Lwt.t =
+    method download_impls (impls:Impl.generic_implementation list) : [ `success | `aborted_by_user ] Lwt.t =
       (* todo: external fetcher on Windows? *)
 
       let zi_impls = ref [] in
       let package_impls = ref [] in
 
       impls |> List.iter (fun impl ->
-        let {Feed_url.feed; Feed_url.id} = Feed.get_id impl in
-        let version = Feed.get_attr_ex FeedAttr.version impl in
+        let {Feed_url.feed; Feed_url.id} = Impl.get_id impl in
+        let version = Impl.get_attr_ex FeedAttr.version impl in
 
         log_debug "download_impls: for %s get %s" (Feed_url.format_url feed) version;
 
         match impl with
-        | {Feed.impl_type = `package_impl info; _} as impl ->
-            if info.Feed.package_state = `installed then
-              log_warning "Package '%s' already installed; skipping" (Feed.get_id impl).Feed_url.id
+        | {Impl.impl_type = `package_impl info; _} as impl ->
+            if info.Impl.package_state = `installed then
+              log_warning "Package '%s' already installed; skipping" (Impl.get_id impl).Feed_url.id
             else
               package_impls := impl :: !package_impls
-        | {Feed.impl_type = `local_impl path; _} -> raise_safe "Can't fetch a missing local impl (%s from %s)!" path (Feed_url.format_url feed)
-        | {Feed.impl_type = `cache_impl info; _} ->
+        | {Impl.impl_type = `local_impl path; _} -> raise_safe "Can't fetch a missing local impl (%s from %s)!" path (Feed_url.format_url feed)
+        | {Impl.impl_type = `cache_impl info; _} ->
             (* Choose the best digest algorithm we support *)
-            if info.Feed.digests = [] then (
-              Q.raise_elem "No digests at all! (so can't choose best) on " impl.Feed.qdom
+            if info.Impl.digests = [] then (
+              Q.raise_elem "No digests at all! (so can't choose best) on " impl.Impl.qdom
             );
-            let digest = Stores.best_digest info.Feed.digests in
+            let digest = Stores.best_digest info.Impl.digests in
 
             (* Pick the first retrieval method we understand *)
-            match info.Feed.retrieval_methods |> U.first_match Recipe.parse_retrieval_method with
+            match info.Impl.retrieval_methods |> U.first_match Recipe.parse_retrieval_method with
             | None -> raise_safe ("Implementation %s of interface %s cannot be downloaded " ^^
                                   "(no download locations given in feed!)") id (Feed_url.format_url feed)
             | Some rm -> zi_impls := (impl, digest, rm) :: !zi_impls
@@ -687,7 +690,7 @@ class external_fetcher command underlying =
     method download_impls impls =
       try_lwt
         let child_nodes = impls |> List.map (function
-          | { qdom; Feed.impl_type = `cache_impl { Feed.digests; _}; _} ->
+          | { qdom; Impl.impl_type = `cache_impl { Impl.digests; _}; _} ->
               let attrs = ref Q.AttrMap.empty in
               digests |> List.iter (fun (name, value) ->
                 attrs := !attrs |> Q.AttrMap.add_no_ns name value
@@ -697,7 +700,7 @@ class external_fetcher command underlying =
               { qdom with
                 Q.child_nodes = manifest_digest :: child_nodes
               }
-          | impl -> impl.Feed.qdom
+          | impl -> impl.Impl.qdom
         ) in
         let root = ZI.make ~child_nodes "interface" in
 
